;+
; NAME:
;     fits2text
;
; PURPOSE:
;     Converts a SpeX spectral FITS file to a text file.
;
; CATEGORY:
;     Spectroscopy
;
; CALLING SEQUENCE:
;     fits2text,filename,CANCEL=cancel
;
; INPUTS:
;     filename = A SpeX spectral FITS file
;
; OPTIONAL INPUTS:
;     None
;
; KEYWORD PARAMETERS:
;     CANCEL - Set on return if there is a problem
;
; OUTPUTS:
;     Writes a text file with the root of the filename+.txt.
;
; OPTIONAL OUTPUTS:
;     None
;
; COMMON BLOCKS:
;     None
;
; SIDE EFFECTS:
;     None
;
; RESTRICTIONS:
;     Must be a SpeX spectral FITS file
;
; PROCEDURE:
;     Easy
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;     2001-09-29 - written M. Cushing, Institute for Astronomy, UH
;-
pro fits2text,filename,CANCEL=cancel

if n_params() ne 1 then begin

    filename = ''
    while filename eq '' do begin

        read, filename,PROMPT='Please enter a SpeX FITS file: '

    endwhile

endif else begin
    
    cancel = cpar('fits2text',filename,1,'Filename',7,0)
    if cancel then return

endelse

filename = strtrim(filename,2)
filename = cfile(filename,CANCEL=cancel)
if cancel then return

readspec,filename,spec,hdr,obsmode,start,stop,norders,naps,orders,$
  xunits,yunits,slith_pix,slith_arc,slitw_pix,slitw_arc,$
  airmass,CANCEL=cancel

npix = fxpar(hdr,'NAXIS1')

pos  = strpos(filename,'.fits')
root = strmid(filename,0,pos)

openw,lun,root+'.txt', /GET_LUN

for i = 0, n_elements(hdr)-1 do printf, lun, hdr[i]

for i = 0, npix[0]-1 do begin
    
    printf, lun,  strjoin( reform(spec[i,*,*],3*naps*norders),'  ' )
    
endfor
close, lun
free_lun, lun

end





