;+
; NAME:
;     gethdrinfo
;
; PURPOSE:
;     Extracts requested hdr info from a FITS header.
;
; CATEGORY:
;     File I/O
;
; CALLING SEQUENCE:
;     result = gethdrinfo(hdr,[keywords],STRING=string,CANCEL=cancel)
;
; INPUTS:
;     hdr      - A FITS header.
;
; OPTIONAL INPUTS:
;     keywords - A string array of keywords to extract.
;
; KEYWORD PARAMETERS:
;     STRING - If a variable is given, gethdrinfo will return a string
;              array of the values of the keywords.
;     CANCEL - Set on return if there is a problem
;    
; OUTPUTS:
;     Returns a structure containing the keywords and their values.
;
; OPTIONAL OUTPUTS:
;     None
;
; COMMON BLOCKS:
;     None
;
; SIDE EFFECTS:
;     None
;
; RESTRICTIONS:
;     None
;
; PROCEDURE:
;     None
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;     2001-08-03 - Written by M. Cushing, Institute for Astronomy, UH
;-
function gethdrinfo,hdr,keywords,STRING=string,CANCEL=cancel
;
cancel = 0

;  Check parameters

if n_params() lt 1 then begin
    
    cancel = 1
    print, 'Syntax - result = gethdrinfo(hdr,keywords,STRING=string,$'
    print, '                             CANCEL=cancel)'
    return,1

endif
cancel = cpar('gethdrinfo',hdr,1,'Hdr',7,1)
if cancel then return,-1

nkeys  = n_elements(keywords)

if nkeys eq 0 then begin

    for i = 0, n_elements(hdr)-1 do begin

        if strmid(hdr[i],8,1) eq '=' then begin

            key = strjoin( strsplit( strtrim( strmid(hdr[i],0,8),2),'-',/EXTR))
            if i eq 0 then keywords = key else begin
                
                z = where(keywords eq key,count)
                if count eq 0 then keywords = (i eq 0) ? key:[keywords,key]

            endelse
        
        endif

    endfor

    keywords = [keywords,'HISTORY']

endif else begin

    cancel = cpar('gethdrinfo',keywords,2,'Keywords',7,[0,1])
    if cancel then return,-1

    
endelse

nkeys = n_elements(keywords)
string = strarr(nkeys)

for i = 0, nkeys-1 do begin

    value = fxpar(hdr,keywords[i])
    if !err lt 0 then value = "Doesn't Exist"
    if (size(value))[0] gt 0 then value = strjoin(value,/SINGLE)
    string[i] = strtrim(value,2)
    key = strjoin( strsplit(keywords[i],'-',/EXTR))
    junk = (i eq 0) ? create_struct(key,value):create_struct(junk,key,value)

endfor

return, junk

end






