;
;******************************************************************************
;
; ------------------------------Event Handlers-------------------------------- 
;
;******************************************************************************
;
pro xmkcals_event, event

widget_control, event.top, GET_UVALUE = state, /NO_COPY
widget_control, event.id,  GET_UVALUE = uvalue
widget_control, /HOURGLASS

case uvalue of

    'Cal Path Button': begin

        path =dialog_pickfile(/DIRECTORY,DIALOG_PARENT=state.w.xmkcals_base,$
                              TITLE='Select Path',/MUST_EXIST)
        
        if path ne '' then begin
            
            widget_control,state.w.calpath_fld[1],SET_VALUE = path
            setfocus, state.w.procpath_fld
            
        endif
        
    end
    
    'Clear Data Path': begin
        
        widget_control, state.w.datapath_fld[1], SET_VALUE=''
        setfocus, state.w.datapath_fld
        
    end
    
    'Clear Proc Path': begin
        
        widget_control, state.w.procpath_fld[1], SET_VALUE=''
        setfocus, state.w.procpath_fld
        
    end
    
    'Clear Cal Path': begin
        
        widget_control, state.w.calpath_fld[1], SET_VALUE=''
        setfocus, state.w.calpath_fld
        
    end
    
    'Clear Table': widget_control, state.w.table,$
      set_value=strarr(1,state.r.tablesize)
    
    'Construct Arc': state.r.doarc = event.value

    'Construct Calibration Images': begin

        xmkcals_setup,state

    end

    'Data Path Button': begin
        
        path= dialog_pickfile(/DIRECTOR,DIALOG_PARENT=state.w.xmkcals_base,$
                              TITLE='Select Path',/MUST_EXIST)
        
        if path ne '' then begin
            
            widget_control,state.w.datapath_fld[1],SET_VALUE = path
            setfocus, state.w.datapath_fld
            
        endif
        
    end
    
    'Combination Statistic': state.r.combinestat = event.value
        
    'Normalize Flat': state.r.normalizeflat = event.value
    
    'Proc Path Button': begin
        
        path= dialog_pickfile(/DIRECTORY,DIALOG_PARENT=state.w.xmkcals_base,$
                              TITLE='Select Path',/MUST_EXIST)
        
        if path ne '' then begin
            
            widget_control,state.w.procpath_fld[1],SET_VALUE = path
            setfocus, state.w.datapath_fld
            
        endif
        
    end
    
    'Quit': begin
        
        widget_control, event.top, /destroy
        goto, getout
        
    end

    'Update Paths': begin

;  Get other paths

        path = cfld(state.w.datapath_fld,7,CANCEL=cancel)
        path = cpath(path,CANCEL=cancel)
        if cancel then begin

            ok = dialog_message('Data path does not exist.',/ERROR,$
                                DIALOG_PARENT=event.id)
            setfocus, state.w.datapath_fld
            goto, cont

        endif
        state.r.datapath = path

        path = cfld(state.w.calpath_fld,7,CANCEL=cancel)
        path = cpath(path,CANCEL=cancel)
        if cancel then begin
            
            ok = dialog_message('Cal path does not exist.',/ERROR,$
                                DIALOG_PARENT=event.id)
            setfocus, state.w.calpath_fld
            goto, cont

        endif
        state.r.calpath = path

        path = cfld(state.w.procpath_fld,7,CANCEL=cancel)
        path = cpath(path,CANCEL=cancel)
        if cancel then begin
            
            ok = dialog_message('Proc path does not exist.',/ERROR,$
                                DIALOG_PARENT=event.id)
            setfocus,state.w.procpath_fld
            goto, cont
            
        endif
        state.r.procpath = path

        home = getenv('HOME')
        openw, lun, home+'/.spextool_paths.txt',/get_lun
        printf, lun, state.r.datapath
        printf, lun, state.r.calpath
        printf, lun, state.r.procpath
        free_lun, lun
        
    end

    else:


endcase

;  Put state variable into the user value of the top level base.
 
cont: 
widget_control, state.w.xmkcals_base, SET_UVALUE=state, /NO_COPY
getout:

end
;
;******************************************************************************
;
; ----------------------------Support procedures------------------------------ 
;
;******************************************************************************
;
pro xmkcals_cleanup,xmkcals_base

widget_control, xmkcals_base, GET_UVALUE = state, /NO_COPY
if n_elements(state) ne 0 then state = 0B
    
end
;
;******************************************************************************
;
pro xmkcals_mkarcs,state

arcinfo = state.r.arcinfo

;  Get flat

for i = 0, state.r.tablesize-1 do begin

    if arcinfo[0,i] eq '' then goto, cont1
    readflat,state.r.calpath+arcinfo[1,i],flat,ncols,nrows,obsmode,slith_pix,$
      slith_arc,slitw_pix,slitw_arc,norders,orders,edgecoeffs,xranges,rms,$
      rotation

    xmkcals_message,state,'Constructing arc in Cal Set '+$
      strcompress(i+1,/RE)+'...'

;  Get arcs

    files = strsplit(arcinfo[0,i],',',/EXTRACT)
    nfiles = n_elements(files)

    pair = (fix(arcinfo[2,i]) eq 0) ? 0:1

    readcmfits,files,arc,hdrinfo,mvar,KEYWORDS=state.r.keywords,$
      ROTATE=state.r.rotation,ITIME=state.r.itime,$
      COADDS=state.r.coadds,NDRS=state.r.ndrs,GAIN=state.r.gain,$
      PAIR=pair,READNOISE=state.r.readnoise,WEIGHTED=0,$
      /UPDATE,WIDGET_ID=state.w.xmkcals_base,CANCEL=cancel
    if cancel then return
    
;  Get sky if necessary

    if arcinfo[3,i] ne '' then begin
        
        sky = strsplit(arcinfo[3,i],',',/EXTRACT)

        if n_elements(sky) gt 1 then begin

            readmfits,sky[0],skya,hdrskya,KEYWORDS=['IRAFNAME'],ROTATE=7,$
              ITIME=state.r.itime,COADDS=state.r.coadds,NDRS=state.r.ndrs,$
              WIDGET_ID=state.w.xmkcals_base,CANCEL=cancel
            if cancel then return
            readmfits,sky[1],skyb,hdrskyb,KEYWORDS=['IRAFNAME'],ROTATE=7,$
              ITIME=state.r.itime,COADDS=state.r.coadds,NDRS=state.r.ndrs,$
              WIDGET_ID=state.w.xmkcals_base,CANCEL=cancel
            if cancel then return
            sky = (skya+skyb)-abs(skya-skyb)
            skyhistory = '  The sky frame is '+strjoin([hdrskya.IRAFNAME,$
                                                       hdrskyb.IRAFNAME],',')
            
        endif else begin
            
            readmfits,sky[0],sky,skyhdr,KEYWORDS=['IRAFNAME'],ROTATE=7,$
              ITIME=state.r.itime,COADDS=state.r.coadds,NDRS=state.r.ndrs,$
              WIDGET_ID=state.w.xmkcals_base,CANCEL=cancel
            if cancel then return
            skyhistory = '  The sky frame is '+strjoin(skyhdr.IRAFNAME)
            
        endelse 
        
        case obsmode of 
        
            'LongXD1.9': arc = concatimg(arc,sky,edgecoeffs,xranges,$
                                         norders,orders,[7,8,9,10],[5,6])
            
            'LongXD2.1': arc = concatimg(arc,sky,edgecoeffs,xranges,$
                                         norders,orders,[7,8,9],[4,5,6])
            
            'LongXD2.3': arc = concatimg(arc,sky,edgecoeffs,xranges,$
                                         norders,orders,[7,8],[4,5,6])

        else:
        
        endcase

    endif else skyhistory = ''

    arc = fixbdpx_median(arc,state.r.bdpxmask,edgecoeffs,norders,xranges,$
                         20,CANCEL=cancel)

    if cancel then return
    xqtv,arc,WID=wid,GROUP_LEADER=state.w.xmkcals_base
    
    junk = tag_exist(hdrinfo[*],state.r.irafname,INDEX=idx)
    
    newhdr = hdrinfo[0]
    newhdr.(idx) = arcinfo[4,i]

    history='This arc was created by mean'+$
      ' combining '+strjoin(hdrinfo[*].(idx), ', ')+'.'+skyhistory
    
    writearc,unrotate(arc,7),state.r.calpath+arcinfo[4,i],newhdr,$
      HISTORY=history

    xmkcals_message,state,'Constructed '+arcinfo[4,i]

    cont1:

endfor

end
;
;******************************************************************************
;
pro xmkcals_message,state,message


widget_control,state.w.message_window, /append,set_value=message

end
;
;******************************************************************************
;
pro xmkcals_mkflats,state

flatinfo = state.r.flatinfo

for i = 0, state.r.tablesize-1 do begin

    if flatinfo[0,i] eq '' then goto,cont2
    
    xmkcals_message,state,'Constructing flat in Cal Set '+$
      strcompress(i+1,/RE)+'...'
    files = strsplit(flatinfo[0,i],',',/EXTRACT)
    nfiles = n_elements(files)

    readmfits,files,data,hdrinfo,var,KEYWORDS=state.r.keywords,$
      ROTATE=state.r.rotation,ITIME=state.r.itime,COADDS=state.r.coadds,$
      NDRS=state.r.ndrs,GAIN=state.r.gain,READNOISE=state.r.readnoise,$
      NIMAGES=nimages,WIDGET_ID=state.w.xmkcals_base,CANCEL=cancel
    if cancel then return

;  Read info from mode_flat.dat
    
    mode = strcompress( (hdrinfo[0]).GRAT,/re )
    readcal,state.r.packagepath+'data/'+mode+'.dat',modename,rotation,$
      norders,orders,start,stop,slith_pix,slith_arc,edgedeg,sg_width,sg_deg,$
      spatdeg,fixed,guesspos,CANCEL=cancel
    if cancel then return
    slitw_arc = float( strmid(hdrinfo[0].SLIT,1,3) )
    slitw_pix = slitw_arc / float(hdrinfo[0].PLATE_SC)

;  Find orders

    findimage = (nfiles ne 1) ? reform(data[*,*,nfiles-1]):data

    xqtv, findimage, WID=wid,GROUP_LEADER=state.w.xmkcals_base

    findorders,findimage,guesspos,start,stop,5,slith_pix,edgedeg,5,$
      edgecoeffs,xranges,WID=wid,CANCEL=cancel

    if nfiles ne 1 then begin

;  Scale the the images (each set of orders) to the same level.

        xmkcals_message,state,'Scaling the flats...'
        
        scalespecsky,data,var,edgecoeffs,xranges,/UPDATE,$
          WIDGET_ID=state.w.xmkcals_base,CANCEL=cancel
        if cancel then return
        
;  Combine the images together.
        
        xmkcals_message, state,'Combining the flats...'
        
        case state.r.combinestat of 
            
            'Weighted Mean': meancomb,data,mean,mvar,DATAVAR=var,$
                                          /UPDATE,$
                                          WIDGET_ID=state.w.xmckcals_base,$
                                          CANCEL=cancel
            'Mean': begin

                delvarx, var
                meancomb,data,mean,mvar,DATAVAR=var,/UPDATE,$
                  WIDGET_ID=state.w.xmkcals_base,$
                  CANCEL=cancel
            
            end
            'Median': medcomb,data,mean,mvar,/UPDATE,$
                              WIDGET_ID=state.w.xmkcals_base,CANCEL=cancel
            
        endcase
        
    endif else mean = data
    if cancel then return
    delvarx,mvar
    delvarx,data

;  Clean the bad pixels

    xmkcals_message, state,'Cleaning bad pixels...'
    
    mean = fixbdpx_median(mean,state.r.bdpxmask,edgecoeffs,norders,$
                          xranges,20,CANCEL=cancel)
    if cancel then return

    xqtv,mean,WID=wid,GROUP_LEADER=state.w.xmkcals_base

;  Normalize the orders if requested.

    if state.r.normalizeflat eq 'Yes' then begin

        xmkcals_message, state,'Normalizing the flat...'
        
        mean = normspecflat(mean,edgecoeffs,norders,xranges,$
                            slith_arc,spatdeg,sg_width,sg_deg,2,RMS=rms,$
                            /UPDATE,WIDGET_ID=state.w.xmkcals_base,$
                            CANCEL=cancel)
        if cancel then return
        
    endif else rms = fltarr(norders)+0.
    
    xqtv, mean,GROUP_LEADER=state.w.xmkcals_base

;  Write the results to disk.

    junk = tag_exist(hdrinfo[*],state.r.irafname,INDEX=idx)
    
    history='This flat was created by 1. scaling '+$
      strjoin(hdrinfo[*].(idx), ', ')+' using the median and then 2. '+$
      state.r.combinestat+' combining the resulting images.'
    
    newhdr = avehdrs(hdrinfo,TIME_OBS=state.r.time,POSANGLE=state.r.posangle,$
                     HA=state.r.ha,AIRMASS=state.r.airmass,CANCEL=cancel)
    if cancel then return
    
    newhdr.(idx) = flatinfo[1,i]

    writeflat,unrotate(mean,7),newhdr,7,fix(orders),edgecoeffs,$
      xranges,slith_pix,slith_arc,slitw_pix,slitw_arc,modename,rms,$
      state.r.calpath+flatinfo[1,i],HISTORY=history
    
    xmkcals_message,state,'Constructed '+flatinfo[1,i]

    cont2:
    
endfor

end
;
;******************************************************************************
;
pro xmkcals_packagepath,state,CANCEL=cancel

widget_control, /HOURGLASS

xmkcals_message,state,'Updating paths and reading bad pixel mask...'

last   = strpos(!path,'Spextool')
first  = strpos(!path,':',last,/REVERSE_SEARCH)
result = strmid(!path,first+1,last-first+7)

path = cpath(result,WIDGET_ID=state.w.xmkcals_base,CANCEL=cancel)
if cancel then return

state.r.packagepath = result

;  Check to see if the path file exists.

home = getenv('HOME')
file = cfile(home+'/.spextool_paths.txt',CANCEL=cancel)
if not cancel then begin

    paths = strarr(3)
    openr, lun, file,/GET_LUN
    readf, lun, paths
    free_lun, lun
    
    widget_control,state.w.datapath_fld[1],SET_VALUE=paths[0]
    widget_control,state.w.calpath_fld[1],SET_VALUE=paths[1]
    widget_control,state.w.procpath_fld[1],SET_VALUE=paths[2]
    
    state.r.datapath = paths[0]
    state.r.calpath  = paths[1]
    state.r.procpath = paths[2]

endif

xmkcals_message,state,'Please load calibration images.'

end
;
;******************************************************************************
;
pro xmkcals_setup,state,CANCEL=cancel

widget_control, state.w.table, get_value=array

flatinfo = strarr(2,state.r.tablesize)
arcinfo  = strarr(5,state.r.tablesize)

;  Make sure all user info is good.

for i = 0, state.r.tablesize-1 do begin

    files = strtrim(array[i],2)
    if files eq '' then goto, cont1

    xmkcals_message,state,'Inspecting Cal Set '+strcompress(i+1,/RE)+'...'

    files = fsextract(files,/INDEX,CANCEL=cancel)
    if cancel then return
    
    fullpaths = mkfullpath(state.r.datapath,files,/INDEX,NI=4,PREFIX='*',$
                           SUFFIX='*.fits',WIDGET_ID=state.w.xmkcals_base,$
                           /EXIST,CANCEL=cancel)
    if cancel then begin

        xmkcals_message,state,'Cal Set '+strcompress(i+1,/RE)+$
          ' does not exist.'
        return

    endif
    fullpaths = findfiles(fullpaths)

    flats = findspexflats(fullpaths,MASK=flatmask,CANCEL=cancel)
 
    if cancel then return
    z = where(flatmask eq 1,count)
    flatoname = 'flat'+strcompress(files[z[0]],/RE)+'-'+$
      strcompress(files[z[count-1]],/RE)+'.fits'
    
    flatinfo[0,i] = strjoin(flats,',')
    flatinfo[1,i] = flatoname
    
    if state.r.doarc eq 'Yes' then begin

        arcs = findspexarcs(fullpaths,MASK=arcmask,AB=ab,CANCEL=cancel)
        if cancel then return
        arcinfo[0,i] = strjoin(arcs,',')
        arcinfo[1,i] = flatoname
        arcinfo[2,i] = (AB eq 0) ? string(0):string(1)

        hdr     = headfits(arcs[0])
        obsmode = strcompress(fxpar(hdr,'GRAT'),/RE)
        if obsmode eq 'LongXD1.9' or obsmode eq 'LongXD2.1' or obsmode eq $
          'LongXD2.3' then begin

            xmkcals_message,state,'Please choose a sky image for Cal Set '+$
              strcompress(i+1,/RE)+'...'

            back:
            sky = dialog_pickfile(DIALOG_PARENT=state.w.xmkcals_base,$
                                  PATH=state.r.datapath,/MUST_EXIST,$
                                  /MULTIPLE_FILES,FILTER='*.fits')
            
            if sky[0] ne '' then begin

                if n_elements(sky) gt 1 then sky = strjoin(sky,',')
                arcinfo[3,i] = sky 

            endif else goto, back

        endif
        z = where(arcmask eq 1,count)
        arconame = 'arc'+strcompress(files[z[0]],/RE)+'-'+$
          strcompress(files[z[count-1]],/RE)+'.fits'        
        arcinfo[4,i] = arconame

    endif

    cont1:

endfor

state.r.arcinfo  = arcinfo
state.r.flatinfo = flatinfo

xmkcals_message,state,'Inspection complete.'

xmkcals_mkflats,state
if state.r.doarc eq 'Yes' then xmkcals_mkarcs,state

end
;
;******************************************************************************
;
pro xmkcals

;  get Spextool path.

last   = strpos(!path,'Spextool')
first  = strpos(!path,':',last,/REVERSE_SEARCH)
result = strmid(!path,first+1,last-first+7)

path = cpath(result,CANCEL=cancel)
if cancel then return

readinstrfile,path+'data/SpeX.dat',instr,irafname,gain,readnoise,$
  itime,coadds,ndrs,time,posangle,ha,airmass,nint,bdpxmk,keywords

;  Set the fonts

textfont   = '-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1'
buttonfont = '-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1'

;  Build three structures which will hold important info.
;  w - contains info pertaining to widget operations.
;  r - contains info pertaining to the reduction process.
;  d - contains all of the data.

w = {calpath_fld:[0L,0L],$
     datapath_fld:[0L,0L],$
     message_window:0L,$
     procpath_fld:[0L,0L],$
     specfiles_fld:[0L,0L],$
     table:0L,$
     xmkcals_base:0L}

tablesize = 12

r = {airmass:airmass,$
     arcinfo:strarr(5,tablesize),$
     bdpxmask:rotate(readfits(path+'data/'+bdpxmk),7),$
     doarc:'Yes',$
     calpath:'',$
     coadds:coadds,$
     datapath:'',$
     combinestat:'Median',$
     flatinfo:strarr(2,tablesize),$
     gain:gain,$
     ha:ha,$
     irafname:irafname,$
     itime:itime,$
     keywords:keywords,$
     ndrs:ndrs,$
     normalizeflat:'Yes',$
     ofileformat:'FITS',$
     packagepath:'',$
     posangle:posangle,$
     procpath:'',$
     readnoise:readnoise,$
     rotation:7,$
     tablesize:tablesize,$
     time:time}


;  Load the three structures in the state structure.

state = {w:w,r:r}

;  Build the widget.

state.w.xmkcals_base = widget_base(title='Make Calibration Images', $
                                     /column)

   quit_button = widget_button(state.w.xmkcals_base,$
                               font=buttonfont,$
                               value='Quit',$
                               uvalue='Quit')

   main_base = widget_base(state.w.xmkcals_base,$
                           /column,$
                           frame=5)

      state.w.message_window = widget_text(main_base, $
                                           FONT=textfont,$
                                           /SCROll,$
                                           YSIZE=4)
      
      path_base = widget_base(main_base,$
                              /column,$
                              /frame)
   
         row = widget_base(path_base,$
                           /row,$
                           /base_align_center)
         
            button = widget_button(row,$
                                   font=buttonfont,$
                                   XSIZE=110,$                             
                                   VALUE='Data Path',$
                                   UVALUE='Data Path Button',$
                                   event_pro = 'xmkcals_event')
            
            path = coyote_field2(row,$
                                 LABELFONT=buttonfont,$
                                 FIELDFONT=textfont,$
                                 TITLE=':',$
                                 VALUE = state.r.datapath,$
                                 UVALUE='Data Path Field',$
                                 XSIZE=60,$
                                 /cr_only,$
                                 event_pro='xmkcals_event',$
                                 textID=textid)
            state.w.datapath_fld = [path,textid]
            
            clear = widget_button(row,$
                                  font=buttonfont,$
                                  VALUE='Clear',$
                                  UVALUE='Clear Data Path',$
                                  event_pro = 'xmkcals_event')
            
         row = widget_base(path_base,$
                           /row,$
                           /base_align_center)
         
            button = widget_button(row,$
                                   font=buttonfont,$
                                   XSIZE=110,$
                                   VALUE='Cal Path',$
                                   UVALUE='Cal Path Button',$
                                   event_pro = 'xmkcals_event')
            
            path = coyote_field2(row,$
                                 LABELFONT=buttonfont,$
                                 FIELDFONT=textfont,$
                                 TITLE=':',$
                                 VALUE=state.r.calpath,$
                                 UVALUE='Cal Path Field',$
                                 XSIZE=60,$
                                 /cr_only,$
                                 event_pro='xmkcals_event',$
                                 textID=textid)
            state.w.calpath_fld = [path,textid]
            
            clear = widget_button(row,$
                                  font=buttonfont,$
                                  VALUE='Clear',$
                                  UVALUE='Clear Cal Path',$
                                  event_pro = 'xmkcals_event')
            
         row = widget_base(path_base,$
                           /row,$
                           /base_align_center)
         
            button = widget_button(row,$
                                   font=buttonfont,$
                                   XSIZE=110,$
                                   VALUE='Proc Path',$
                                   UVALUE='Proc Path Button',$
                                   event_pro = 'xmkcals_event')
            
            path = coyote_field2(row,$
                                 LABELFONT=buttonfont,$
                                 FIELDFONT=textfont,$
                                 TITLE=':',$
                                 UVALUE='Proc Path Field',$
                                 VALUE=state.r.procpath,$
                                 XSIZE=60,$
                                 /cr_only,$
                                 event_pro='xmkcals_event',$
                                 textID=textid)
            state.w.procpath_fld = [path,textid]
            
            clear = widget_button(row,$
                                  font=buttonfont,$
                                  VALUE='Clear',$
                                  UVALUE='Clear Proc Path',$
                                  event_pro = 'xmkcals_event')
            
         load = widget_button(path_base,$
                              VALUE='Update Paths',$
                              font=buttonfont,$
                              UVALUE='Update Paths')

      cals_base = widget_base(main_base,$
                              /row,$
                              /frame)

         col1_base = widget_base(cals_base,$
                                 /column)

            construct = cw_bgroup(col1_base,$
                                  font=buttonfont,$
                                  ['Yes','No'],$
                                  /row,$
                                  /return_name,$
                                  /exclusive,$
                                  label_left='Construct Arc:',$
                                   UVALUE='Construct Arc',$
                                  set_VALUE=0)            
               
            combine_bg = cw_bgroup(col1_base,$
                                   font=buttonfont,$
                                   ['Weighted Mean','Mean','Median'],$
                                   /row,$
                                   /return_name,$
                                   /no_release,$
                                   /exclusive,$
                                   label_top='Combination Statistic:',$
                                   UVALUE='Combination Statistic',$
                                   SET_VALUE=2)
            
            normal_bg = cw_bgroup(col1_base,$
                                  font=buttonfont,$
                                  ['Yes','No'],$
                                  /row,$
                                  /return_name,$
                                  /no_release,$
                                  /exclusive,$
                                  label_left='Normalize Flat:',$
                                  UVALUE='Normalize Flat',$
                                  set_VALUE=0)
            
         col2_base = widget_base(cals_base,$
                                 /column)

               clear_button = widget_button(col2_base,$
                                            font=buttonfont,$
                                            VALUE='Clear Table',$
                                            UVALUE = 'Clear Table')

               rowlab='Cal Set '+strcompress(indgen(state.r.tablesize)+1,/RE)
               state.w.table = widget_table(col2_base,$
                                            column_label=['File Numbers'],$
                                            row_label = rowlab,$
                                            column_widths=[250],$
                                            xsize=1,$
                                            ysize=state.r.tablesize,$
                                            alignment=1,$
                                            value=strarr(1,state.r.tablesize),$
                                            /editable,$
                                            uvalue='Table')
               
               combine_button = widget_button(main_base,$
                                      font=buttonfont,$
                                      VALUE='Construct Calibration Images',$
                                      UVALUE = 'Construct Calibration Images')

; Get things running.  Center the widget using the Fanning routine.

centertlb,state.w.xmkcals_base

widget_control, state.w.xmkcals_base, /realize

; Start the Event Loop. This will be a non-blocking program.

XManager, 'xmkcals', $
  state.w.xmkcals_base, $
  /No_Block,$
  cleanup = 'xmkcals_cleanup'

;  Load package path

xmkcals_packagepath,state

; Put state variable into the user value of the top level base.

widget_control, state.w.xmkcals_base, set_uvalue=state, /no_copy

end






